.transfer.button.button-secondary.bicolor {
    overflow: visible;
    border: 1px solid #00676a;
    width: auto;
    color: #007d7e;
    height: 3em;
}

.transfer.button.button-secondary.bicolor::before {
    background-color: #00676a;
    color: #fff;
    border-radius: 0px;
}

.transfer.chooser.page-chooser .unchosen::before {
    content: none;
}

.transfer.chooser.page-chooser .chosen::before {
    vertical-align: top;
}

@media(min-width:950px) {
    .transfer.chooser-parent {
        display: flex;
    }
}

button.link {
	font-size: 0.85em;
	background: none;
	border: none;
	color: #007d7e;
	text-transform: uppercase;
}

ul.transfer.actions {
    list-style-type: none;
    margin: 0;
}

.transfer.chooser.page-chooser .chosen {
    border: 1px solid #e6e6e6;
    border-radius: 6px;
    padding: 15px 15px 15px 15px;
    color: #666;
    max-width: 500px;
}

.transfer.subtitle {
    margin: 0;
    font-size: 0.85em;
}

.transfer.title-wrapper {
    display: inline-block;
}

.chosen .transfer.title {
    margin: 0;
    padding: 0;
}

.transfer.chooser {
    margin: 15px 15px 15px 0px;
}

ol.transfer.numbered {
  counter-reset: section;
  list-style: none;
}

li.transfer.numbered {
  margin: 0 0 10px 0;
  line-height: 30px;
}

li.transfer.numbered::before {
  content: counter(section);
  counter-increment: section;
  display: inline-block;
  width: 30px;
  height: 30px;
  margin: 0 20px 0 0;
  border: 1px solid #00676a;
  border-radius: 100%;
  text-align: center;
  font-size: 20px;
  background-color: #00676a;
  color: #fff;
}

.transfer.list-container {
    display: inline-block;
}

.transfer.button.close {
    color: var(--w-color-black);
}
