{% extends "wagtailadmin/base.html" %}
{% load wagtailimages_tags wagtailadmin_tags i18n l10n %}
{% block titletag %}{% trans "Choose a page or snippet" %}{% endblock %}
{% block extra_css %}
    {{ block.super }}

    <link rel="stylesheet" href="{% versioned_static 'wagtail_transfer/css/transfer-styles.css' %}">
{% endblock %}

{% block extra_js %}
    {{ block.super }}

    <script src="{% versioned_static 'wagtail_transfer/js/wagtail_transfer.js' %}"></script>
{% endblock %}

{% block content %}
    {% trans "Choose a page or snippet" as title_str %}
    {% include "wagtailadmin/shared/header.html" with title=title_str icon="doc-empty-inverse" %}

    <div class="nice-padding">
        <div data-wagtail-component="content-import-form" data-local-api-base-url="{% url 'wagtail_transfer_admin:page_chooser_api:pages:listing' %}" data-local-check-uid-url="{% url 'wagtail_transfer_admin:check_uid' %}" data-sources="{{ sources_data }}" data-action="{% url 'wagtail_transfer_admin:import' %}" data-csrf-token="{{ csrf_token }}"></div>
    </div>
{% endblock %}
