# Generated by Django 2.2.7 on 2019-12-05 19:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_transfer', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ImportedFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(upload_to='')),
                ('source_url', models.URLField(max_length=1000)),
                ('hash', models.CharField(max_length=40)),
                ('size', models.PositiveIntegerField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]
