# Generated by Django 2.2.6 on 2019-10-15 01:31

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='IDMapping',
            fields=[
                ('uid', models.UUIDField(primary_key=True, serialize=False)),
                ('local_id', models.CharField(max_length=255)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'unique_together': {('content_type', 'local_id')},
            },
        ),
    ]
